import logging
logging.basicConfig(level=logging.INFO)
from azuredevopsX.abstractdevops import AbstractDevOps

# TODO
class Identity(AbstractDevOps):
	
	def __init__(self,personal_access_token, organization_url):
		super(Identity,self).__init__(personal_access_token=personal_access_token,organization_url=organization_url)



import logging
logging.basicConfig(level=logging.INFO)
from azuredevopsX.abstractdevops import AbstractDevOps
from azure.devops.v5_1.work_item_tracking import WorkItemTrackingClient 
from azure.devops.v5_1.work_item_tracking.models import Wiql
from azuredevopsX import factories

# Represent a item of work in a project xxx

class WorkItem(AbstractDevOps):

	def __init__(self,personal_access_token, organization_url):
		super(WorkItem,self).__init__(personal_access_token=personal_access_token,organization_url=organization_url)
	

	def get_all (self, today=False):
		try:
			logging.info("Start function: get_work_items")
			
			work_items_list = []
			work_items = None
			if today:
				work_items = self.get_query_today()
			else:
				work_items = self.get_work_item_query_by_wiql()
			
			# Dividindo em vetores de no máximo 200 itens. O serviço permite no máximo 200 ids por requisição;
			work_items_list_split = [work_items[x:x+200] for x in range(0,len(work_items), 200)]

			for work_items_list_item in work_items_list_split:
				
				work_item_list_retrieve = []
				#criando o vertor a ser enviado
				for work_item in work_items_list_item:
					work_item_list_retrieve.append(int(work_item.id))
				
				work_item_retrieved = self.get_work_items(ids=[*work_item_list_retrieve])				
				
				work_items_list.extend(work_item_retrieved)
			
			logging.info("End function: get_work_items")
			return work_items_list

		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 

	

	# Return a specifc workitem from a project with detais
	def get_work_items(self,ids):
		try:
			logging.info("Start function: get_work_items")
			result = self.work_item_tracking_client.get_work_items(ids=ids, 
                                                        project=None,                                                       
                                                       expand="All")
			logging.info("End function: get_work_items")

			list_result = []
			
			for item in result:
				item._links = item._links.__dict__

				if item.comment_version_ref is not None:
					item.comment_version_ref = item.comment_version_ref.__dict__

				if item.relations is not None:
					item_relations = []
					
					for relation in item.relations:
						item_relations.append(relation.__dict__)
					
					item.relations = item_relations
			
				list_result.append(item)
			return list_result
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 
		
	
	# Return a specifc workitem from a project with detais
	def get_work_item(self,work_item_id):
		try:
			logging.info("Start function: get_work_item")
			logging.info("End function: get_work_item")

			return self.work_item_tracking_client.get_work_item(id=work_item_id, 
                                                        project=None,                                                       
                                                       expand="All")
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 


	def get_relation(self, relation):
		try:
			logging.info("Start function: get_work_item")
			logging.info("End function: get_work_item")

			return self.work_item_tracking_client.get_relation_type(relation)
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 
    
	def get_backlog_work_items(self, project,team, backlog):
		try:
			logging.info("Start function: get_work_item")
			logging.info("End function: get_work_item")
			team_context = self.create_team_context(project, team)
			return self.work_client.get_backlog_level_work_items(team_context, backlog.id)
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 

	def get_work_item_query_by_wiql(self):
		try:
			wiql = Wiql(
			query="""select 
					*
					from WorkItems"""
					
				)
			wiql_results = self.work_item_tracking_client.query_by_wiql(wiql).work_items
			if wiql_results:
				return wiql_results
			else:
				return []
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 	

	def get_query_today(self):
		try:
			wiql = Wiql(
			query="""select 
					*
					from WorkItems
					where 
					[System.CreatedDate] = @Today 
					OR [System.ChangedDate] = @Today 
					OR [Microsoft.VSTS.Common.ClosedDate] = @Today 
					OR [Microsoft.VSTS.Common.ResolvedDate] = @Today
					OR [System.RevisedDate] = @Today
					OR [Microsoft.VSTS.Common.StateChangeDate]= @Today
					order by [System.ChangedDate] desc"""
				)
			wiql_results = self.work_item_tracking_client.query_by_wiql(wiql).work_items
			if wiql_results:
				return wiql_results
			else:
				return []
		
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 
	
	
	
	def get_work_item_query_by_wiql_epic_user_story_product_backlog_item(self):
		try:
			wiql = Wiql(
			query="""select 
					*
					from WorkItems
					order by [System.ChangedDate] desc"""
					
				)
			wiql_results = self.work_item_tracking_client.query_by_wiql(wiql).work_items
			if wiql_results:
				return wiql_results
			else:
				return []
		
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 

	def get_work_item_query_by_wiql_task(self):
		try:
			wiql = Wiql(
			query="""
					select [System.Id],
						[System.WorkItemType],
						[System.Title],
						[System.State],
						[System.AreaPath],
						[System.IterationPath],
						[System.Tags],
						[System.TeamProject]
					from WorkItems
					where 
					[System.WorkItemType] = 'Task' 
					order by [System.ChangedDate] desc"""
				)
			wiql_results = self.work_item_tracking_client.query_by_wiql(wiql).work_items
			if wiql_results:
				return wiql_results
			else:
				return []

		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 

	def get_work_item_query_by_wiql(self):
		try:
			wiql = Wiql(
			query="""
					select [System.Id],
						[System.WorkItemType],
						[System.Title],
						[System.State],
						[System.AreaPath],
						[System.IterationPath],
						[System.Tags]
					from WorkItems
					order by [System.ChangedDate] desc"""
				)
			wiql_results = self.work_item_tracking_client.query_by_wiql(wiql).work_items
			if wiql_results:
				return wiql_results
			else:
				return []
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 


import factory
from .project import Project
from .backlog import Backlog
from .identity import Identity
from .interaction import Interaction
from .status import Status
from .team import Team
from .teammember import TeamMember
from .workitem import WorkItem
from .workitemtype import WorkItemType

class ProjectFactory(factory.Factory):
    
    class Meta:
        model = Project
        
    personal_access_token = None
    organization_url = None

class BacklogFactory(factory.Factory):
    
    class Meta:
        model = Backlog
    personal_access_token = None
    organization_url = None

class IdentityFactory(factory.Factory):
    
    class Meta:
        model = Identity
    personal_access_token = None
    organization_url = None

class InteractionFactory(factory.Factory):
    
    class Meta:
        model = Interaction
    personal_access_token = None
    organization_url = None

class StatusFactory(factory.Factory):
    
    class Meta:
        model = Status
    personal_access_token = None
    organization_url = None

class TeamFactory(factory.Factory):
    
    class Meta:
        model = Team
    personal_access_token = None
    organization_url = None

class TeamMemberFactory(factory.Factory):
    
    class Meta:
        model = TeamMember
    personal_access_token = None
    organization_url = None


class WorkItemFactory(factory.Factory):
    
    class Meta:
        model = WorkItem
    personal_access_token = None
    organization_url = None


class WorkItemTypeFactory(factory.Factory):
    
    class Meta:
        model = WorkItemType
    personal_access_token = None
    organization_url = None


import logging
logging.basicConfig(level=logging.INFO)
from azuredevopsX.abstractdevops import AbstractDevOps
from azuredevopsX import factories
# Represent a type of Workitem 
class WorkItemType(AbstractDevOps):
	
	def __init__(self,personal_access_token, organization_url):
		super(WorkItemType,self).__init__(personal_access_token=personal_access_token,organization_url=organization_url)
	

	def get_all(self, today = False):
		try:
			logging.info("Start function: get_work_item")
			logging.info("End function: get_work_item")
			project_service = factories.ProjectFactory(personal_access_token=self.personal_access_token,organization_url=self.organization_url)
			projects = project_service.get_all()
			result = []
			for project in projects:
				
				workitems = self.work_item_tracking_client.get_work_item_types(project.id)
				print (workitems)
				for workitem in workitems:
					workitem.additional_properties["project"] = project.__dict__
					result.append (workitem)
			return result
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 


	# Return workitem types
	def get_work_item_type(self,project):
		try:
			logging.info("Start function: get_work_item")
			logging.info("End function: get_work_item")
			return self.work_item_tracking_client.get_work_item_types(project)
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 


from azure.devops.credentials import BasicAuthentication
from azure.devops.connection import Connection
from azure.devops.v5_1.work.models import TeamContext
from azure.devops.v5_1.client_factory import ClientFactoryV5_1
from azure.devops.v5_1.work_item_tracking import WorkItemTrackingClient 
from azure.devops.v5_1.work_item_tracking.models import Wiql
from pprint import pprint
from abc import ABC

class AbstractDevOps(ABC):

    def __init__(self, personal_access_token, organization_url):

        self.personal_access_token = personal_access_token
        self.organization_url = organization_url

        self.credentials = BasicAuthentication('', personal_access_token)
        self.connection = Connection(base_url=organization_url, creds=self.credentials)

        self.core_client = self.connection.clients.get_core_client()
        self.work_client = self.connection.clients_v5_1.get_work_client()
        self.work_item_tracking_client = self.connection.clients_v5_1.get_work_item_tracking_client()
        self.work_item_process_tracking_client = self.connection.clients_v5_1.get_work_item_tracking_process_client()
    
    def create_team_context(self, project, team):
        return  TeamContext(project, project.id, team, team.id)
        
    


import logging
logging.basicConfig(level=logging.INFO)
from azuredevopsX import factories
from azuredevopsX.abstractdevops import AbstractDevOps

# Represent the time-box in a project 
class Interaction(AbstractDevOps):
	def __init__(self,personal_access_token, organization_url):
		super(Interaction,self).__init__(personal_access_token=personal_access_token,organization_url=organization_url)
		

	def get_interactions(self, project, team):
		try:
			logging.info("Start function: get_work_items")
			team_context = self.create_team_context(project, team)
			logging.info("End function: get_work_items")
			return self.work_client.get_team_iterations(team_context)
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__)
	
	def get_all(self, today = False):

		try:
			logging.info("Start function: get_all_interaction")
			interactions = []
			
			project_service = factories.ProjectFactory(personal_access_token=self.personal_access_token,organization_url=self.organization_url)
			team_service = factories.TeamFactory(personal_access_token=self.personal_access_token,organization_url=self.organization_url)
			projects = project_service.get_all()
			
			for project in projects:
				teams = team_service.get_teams(project.id)
				for team in teams:
					team_context = self.create_team_context(project, team)
					for interaction in self.work_client.get_team_iterations(team_context):
						
						interaction.additional_properties["project"] = project.__dict__
						interaction.additional_properties["team"] = team.__dict__
						interaction.attributes = interaction.attributes.__dict__
						interactions.append (interaction)
			
			logging.info("End function: get_all_interaction")
			return interactions
			
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__)
	
	


import logging
logging.basicConfig(level=logging.INFO)
from azuredevopsX.abstractdevops import AbstractDevOps
from azuredevopsX import factories
# Represents the teams in a project
class Team(AbstractDevOps):
	
	def __init__(self,personal_access_token, organization_url):
		super(Team,self).__init__(personal_access_token=personal_access_token,organization_url=organization_url)
	
	def get_teams(self, project_id):
		return self.core_client.get_teams(project_id)

	def get_all(self, today = False):
		try:

			project_service = factories.ProjectFactory(personal_access_token=self.personal_access_token,organization_url=self.organization_url)
			projects = project_service.get_all()
			teams = []
			for project in projects:
				for team in self.get_teams(project_id = project.id):
					team.additional_properties["project"] = project.__dict__
					teams.append(team)
			
			return teams				

		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 



import logging
logging.basicConfig(level=logging.INFO)
from azuredevopsX.abstractdevops import AbstractDevOps
from azuredevopsX import factories

# Represents a Backlog of project and Team
class Backlog(AbstractDevOps):

	def __init__(self,personal_access_token, organization_url):
		super(Backlog,self).__init__(personal_access_token=personal_access_token,organization_url=organization_url)
	
		
	# Return a backlog from a project and team
	def get_backlog(self,project,team): 
		try:
			logging.info("Start function: get_backlog")
			team_context = self.create_team_context(project, team)
			logging.info("End function: get_backlog")
			return self.work_client.get_backlogs(team_context)
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 
	
	def get_all(self,today = False):
		try:
			logging.info("Start function: get_backlog")
			
			project_service = factories.ProjectFactory(personal_access_token=self.personal_access_token,organization_url=self.organization_url)
			team_service = factories.TeamFactory(personal_access_token=self.personal_access_token,organization_url=self.organization_url)
			projects = project_service.get_all()
			backlogs = []
			for project in projects:
				teams = team_service.get_teams(project.id)
				for team in teams:
					team_context = self.create_team_context(project, team)		
					logging.info("End function: get_backlog")
					backlogs.extend (self.work_client.get_backlogs(team_context))
			
			return backlogs
		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__)

	


import logging
logging.basicConfig(level=logging.INFO)
from azuredevopsX.abstractdevops import AbstractDevOps

# Represents a software Project
class Project(AbstractDevOps):

	def __init__(self,personal_access_token, organization_url):
		super(Project,self).__init__(personal_access_token=personal_access_token,organization_url=organization_url)
	
	# Returns a list of project
	def get_all(self, today=False): 
		projects = []
		try:
			logging.info("Start function: get_projects")
			
			get_projects_response = self.core_client.get_projects()
			
			while get_projects_response is not None:
				
				for project in get_projects_response.value:
					projects.append(project)
				if get_projects_response.continuation_token is not None and get_projects_response.continuation_token != "":
					# Get the next page of projects
					get_projects_response = self.core_client.get_projects(continuation_token=get_projects_response.continuation_token)
				else:
					# All projects have been retrieved
					get_projects_response = None

		except Exception as e: 
			logging.error("OS error: {0}".format(e))
			logging.error(e.__dict__) 

		logging.info("Retrieve All Projects")
		
		return projects

	def get_processes(self):
		return self.work_item_process_tracking_client.get_list_of_processes()

	def get_process_behaviors(self, process_id):
		return self.work_item_process_tracking_client.get_process_behaviors(process_id)
    
	def get_process_by_its_id(self, process_type_id):
		return self.work_item_process_tracking_client.get_process_by_its_id(process_type_id)
    
	def get_state_definitions(self, process_id):
		return self.work_item_process_tracking_client.get_state_definitions(process_id)


import logging
logging.basicConfig(level=logging.INFO)
from azuredevopsX.abstractdevops import AbstractDevOps
from azuredevopsX import factories

# Represent a team member
class TeamMember(AbstractDevOps):

	def __init__(self,personal_access_token, organization_url):
		super(TeamMember,self).__init__(personal_access_token=personal_access_token,organization_url=organization_url)
	
	def get_team_members(self, project_id, team_id):
		return self.core_client.get_team_members_with_extended_properties(project_id=project_id,team_id=team_id)
    
	def get_all(self, today = False):
		project_service = factories.ProjectFactory(personal_access_token=self.personal_access_token,organization_url=self.organization_url)
		projects = project_service.get_all()
			
		all_team_members = []
        
		for project in projects:
			teams = self.core_client.get_teams(project.id)
			for team in teams:
				team_members = self.get_team_members(project.id,team.id)
				for team_member in team_members:
					team_member.additional_properties["project"] = project.__dict__
					team_member.additional_properties["team"] = team.__dict__
					team_member.identity = team_member.identity.__dict__
					team_member.identity["_links"] = team_member.identity["_links"].__dict__
					all_team_members.append(team_member)
		
		return all_team_members


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='azuredevopsX',  # Required
    version='0.0.17',  # Required
    author="Paulo Sérgio dos Santos Júnior",
    author_email="paulossjunior@gmail.com",
    description="DevOps é um produto da Microsoft que fornece controle de versão, relatórios, gerenciamento de requisitos, gerenciamento de projetos, compilações automatizadas, gerenciamento de laboratório, recursos de gerenciamento de testes e versões.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/integration_seon/libs/application/tfs",
    packages=find_packages(),
    
    install_requires=[
        'azure-devops',
        'requestx',
        'factory-boy'
    ],

    classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
    setup_requires=['wheel'],
    
)


